@@ -1,4 +1,4 @@
-// $Id: DB2Backend.cc 1985 2011-02-07 09:39:22Z ahu $
+// $Id$
 
 #include <string>
 #include <unistd.h>
@@ -7,13 +7,13 @@
 
 #include "pdns/namespaces.hh"
 
-#include <dns.hh>
-#include <dnsbackend.hh>
-#include <dnspacket.hh>
-#include <ueberbackend.hh>
-#include <ahuexception.hh>
-#include <logger.hh>
-#include <arguments.hh>
+#include <pdns/dns.hh>
+#include <pdns/dnsbackend.hh>
+#include <pdns/dnspacket.hh>
+#include <pdns/ueberbackend.hh>
+#include <pdns/ahuexception.hh>
+#include <pdns/logger.hh>
+#include <pdns/arguments.hh>
 
 #include "DB2Exception.hh"
 #include "DB2Backend.hh"
