@@ -47,23 +47,18 @@ void CommunicatorClass::retrievalLoopThread(void)
       sr=d_suckdomains.front();
       d_suckdomains.pop_front();
     }
-    try {
-      suck(sr.domain,sr.master);
-    }
-    catch(AhuException& ae) {
-      cerr<<"Error: "<<ae.reason<<endl;
-    }
+    suck(sr.domain,sr.master);
   }
 }
 
-
 void CommunicatorClass::go()
 {
   pthread_t tid;
-  pthread_create(&tid,0,&launchhelper,this);
+  pthread_create(&tid,0,&launchhelper,this); // Starts CommunicatorClass::mainloop()
   for(int n=0; n < ::arg().asNum("retrieval-threads"); ++n)
-    pthread_create(&tid, 0, &retrieveLaunchhelper, this);
+    pthread_create(&tid, 0, &retrieveLaunchhelper, this); // Starts CommunicatorClass::retrievalLoopThread()
 
+  d_preventSelfNotification =::arg().mustDo("prevent-self-notification");
 }
 
 void CommunicatorClass::mainloop(void)
