@@ -1,24 +1,24 @@
-alter table records add ordername	VARCHAR(255);
+alter table records add ordername   VARCHAR(255);
 alter table records add auth bool;
-create index orderindex on records(ordername);
+create index recordorder on records (domain_id, ordername text_pattern_ops);
 
 create table domainmetadata (
- id		SERIAL PRIMARY KEY,
- domain_id	INT REFERENCES domains(id) ON DELETE CASCADE,
- kind		VARCHAR(16),
- content	TEXT
+ id         SERIAL PRIMARY KEY,
+ domain_id  INT REFERENCES domains(id) ON DELETE CASCADE,
+ kind       VARCHAR(16),
+ content    TEXT
 );
 
 create index domainidmetaindex on domainmetadata(domain_id);               
 
 
 create table cryptokeys (
- id		SERIAL PRIMARY KEY,
- domain_id	INT REFERENCES domains(id) ON DELETE CASCADE,
- flags		INT NOT NULL,
- active		BOOL,
- content	TEXT
-);		 
+ id         SERIAL PRIMARY KEY,
+ domain_id  INT REFERENCES domains(id) ON DELETE CASCADE,
+ flags      INT NOT NULL,
+ active     BOOL,
+ content    TEXT
+);       
 create index domainidindex on cryptokeys(domain_id);
 
 
@@ -28,10 +28,10 @@ create index domainidindex on cryptokeys(domain_id);
 -- GRANT ALL ON cryptokeys_id_seq TO pdns;
 
 create table tsigkeys (
- id		SERIAL PRIMARY KEY,
- name		VARCHAR(255),
- algorithm	VARCHAR(50), 
- secret		VARCHAR(255)
+ id         SERIAL PRIMARY KEY,
+ name       VARCHAR(255),
+ algorithm  VARCHAR(50), 
+ secret     VARCHAR(255)
 );
 
 create unique index namealgoindex on tsigkeys(name, algorithm);
