@@ -64,6 +64,14 @@ bool Utility::setBlocking(sock_t sock)
   return true;
 }
 
+bool Utility::setCloseOnExec(sock_t sock)
+{
+  int flags=fcntl(sock,F_GETFD,0);    
+  if(flags<0 || fcntl(sock, F_SETFD,flags|FD_CLOEXEC) <0)
+    return false;
+  return true;
+}
+
 const char *Utility::inet_ntop(int af, const char *src, char *dst, size_t size)
 {
   return ::inet_ntop(af,src,dst,size);
@@ -91,6 +99,19 @@ void Utility::dropPrivs( int uid, int gid )
     else
       theL()<<Logger::Info<<"Set effective group id to "<<gid<<endl;
 
+    struct passwd *pw=getpwuid(uid);
+    if(!pw) {
+      theL()<<Logger::Warning<<"Unable to determine user name for uid "<<uid<<endl;
+      if (setgroups(0, NULL)<0) {
+        theL()<<Logger::Critical<<"Unable to drop supplementary gids: "<<stringerror()<<endl;
+        exit(1);
+      }
+    } else {
+      if (initgroups(pw->pw_name, gid)<0) {
+        theL()<<Logger::Critical<<"Unable to set supplementary groups: "<<stringerror()<<endl;
+        exit(1);
+      }
+    }
   }
 
   if(uid) {
@@ -216,7 +237,7 @@ time_t Utility::timegm(struct tm *const t)
   if (t->tm_sec>60) { t->tm_min += t->tm_sec/60; t->tm_sec%=60; }
   if (t->tm_min>60) { t->tm_hour += t->tm_min/60; t->tm_min%=60; }
   if (t->tm_hour>60) { t->tm_mday += t->tm_hour/60; t->tm_hour%=60; }
-  if (t->tm_mon>12) { t->tm_year += t->tm_mon/12; t->tm_mon%=12; }
+  if (t->tm_mon>11) { t->tm_year += t->tm_mon/12; t->tm_mon%=12; }
  
   while (t->tm_mday>spm[1+t->tm_mon]) {
     if (t->tm_mon==1 && isleap(t->tm_year+1900)) {
